
###############
## load data ##
###############

# load binary data:
load(file = "meta.data.rda")

# alternatively, load data from text file
d <- read.table(file = "meta.data.txt", header = TRUE)

###############
## prepare R ##
###############

# load packages necessary for the analysis:
require(BayesFactor)  # does the calculation of the Bayes Factor
require(snowfall)   # required for using multiple cores

nCPU <- 6  # number of cpus available
sfInit( parallel=TRUE, cpus=nCPU, type = "SOCK" )   # initialize snowfall
sfLibrary(BayesFactor)  # load BayesFactor ate each child process (i.e., core)

#################################
## set parameters for analysis ##
#################################

# which method to use for obtaining the marginal likelihood
method <- "laplace"
iterations <- 10000   # irrelevant for laplace ("laplace does not sample")

rscaleFixed <- 0.5    # scaling factor for g-priors for fixed effects
# change this to 0.2 for alternative analysis

whichRandom <- c("id", "content")  # specify random effects
# change this to c("id", "content", "exp") for alternative analysis


###################
## main analysis ##
###################

### make model formulas:

randomEffects <- c(
# id, exp; no slopes: 
"exp + id",
# id, exp, content; slopes w/o validity: 
"exp + exp:believability + id + id:believability",
# id, exp, content; full slopes: 
"exp + exp:believability + exp:validity + exp:validity:believability + id + id:believability + id:validity + id:believability:validity",
# id, exp, content; no slopes: 
"exp + id + content",
# id, exp, content; slopes w/o validity: 
"exp + exp:believability + id + id:believability + content",
# id, exp, content; full slopes: 
"exp + exp:believability + exp:validity + exp:validity:believability + id + id:believability + id:validity + id:believability:validity + content + validity:content")

fixedEffects <- c(
    "resp ~ ",
    "resp ~ believability + ",
    "resp ~ validity + ",
    "resp ~ validity + believability + ",
    "resp ~ validity * believability + " 
)

# create vector containing all model formulas:
all.models.f <- lapply(as.vector(outer(fixedEffects, randomEffects, FUN = paste0)), as.formula)


### calculate all models in one step:

all.models <- sfLapply(all.models.f, lmBF, whichRandom = whichRandom, data = d, method = method, iterations = iterations, rscaleFixed = rscaleFixed)
# to run the analysis without using multicore use lapply instead of sfLapply
# to increase the speed of the analysis, use sfClusterApplyLB instead of sfLapply


########################
## make results table ##
########################

bf.table <- c(do.call("c", all.models))  # concatenate objects
bf.table <- bf.table[-1]/bf.table[1]  # use first model as denominator

# names of models:
effects <- c("none", "believability", "validity", "validity + believability", "validity $\\times$ believability")
col1 <- rep(effects, 6)[-1]

# make data.frame
t.out <- data.frame("Fixed Effects" = col1, "BF" = as.data.frame(bf.table)$bf, "log(BF)" = log(as.data.frame(bf.table)$bf), check.names = FALSE)
t.out

## for nice printing use either ascii or xtable (for LaTeX output)

require(ascii)
print(ascii(t.out, include.rownames = FALSE, format = c("d", "E", "f"), digits = c(2, 2, 2)), type = "org")

require(xtable)
print(xtable(t.out, display = c("f", "f", "e", "f")), type = "latex", sanitize.text.function = identity)

###################################
## analysis of posterior samples ##
###################################

# obtain samples from the posterior distribution of M29
m29 <- posterior(all.models[[30]], iterations = 1000)

# names of parameters in the model
vars <- varnames(mcmc(m29))
vs <- varnames(mcmc(m29))


### fixed effects figure:

# credible intervals and means:
hpds <- HPDinterval(mcmc(m29)[,vs[2:6]])
fixed.means <- apply(mcmc(m29)[,vs[2:6]], 2, mean)

# make figure
pdf("figure2.pdf", width = 7, height = 8)

xlim.dens  <- c(-1, 1)
ylim.dens <- c(0, 6)
pos.mean <- -0.115
pch.mean  <- 4
show.obs <- TRUE
col.zero <- "darkgrey"

par(mfrow = c(2, 2))

par(mar = c(2, 4, 7, 2) + 0.1)
which <- 3
densplot(mcmc(m29)[,vs[which]], main = "", xlim = xlim.dens, ylim = ylim.dens, xlab = "", show.obs = show.obs)
title("Validity", line = 1)
mtext("Density of posterior samples for fixed effects", font = 2, at = c(14, 1.5), line = 4, cex = 1.5)
abline(h = 0, col = col.zero)
abline(v = hpds[vs[which], 1])
abline(v = hpds[vs[which], 2])
points(fixed.means[vs[which]], pos.mean, pch = pch.mean)

which <- 4
densplot(mcmc(m29)[,vs[which]], main = "", xlim = xlim.dens, ylim = ylim.dens, xlab = "", show.obs = show.obs)
title("Believability: Abstract", line = 1)
abline(h = 0, col = col.zero)
abline(v = hpds[vs[which], 1])
abline(v = hpds[vs[which], 2])
points(fixed.means[vs[which]], pos.mean, pch = pch.mean)

par(mar = c(4, 4, 5, 2) + 0.1)
which <- 5
densplot(mcmc(m29)[,vs[which]], main = "", xlim = xlim.dens, ylim = ylim.dens, xlab = "", show.obs = show.obs)
title("Believability: Believable", line = 1)
abline(h = 0, col = col.zero)
abline(v = hpds[vs[which], 1])
abline(v = hpds[vs[which], 2])
points(fixed.means[vs[which]], pos.mean, pch = pch.mean)

which <- 6
densplot(mcmc(m29)[,vs[which]], main = "", xlim = xlim.dens, ylim = ylim.dens, xlab = "", show.obs = show.obs)
title("Believability: Unbelievable", line = 1)
abline(h = 0, col = col.zero)
abline(v = hpds[vs[which], 1])
abline(v = hpds[vs[which], 2])
points(fixed.means[vs[which]], pos.mean, pch = pch.mean)

dev.off()


### Random effects figure:

pdf("figure3.pdf", width = 7, height = 8)

xlim.rand  <- c(-0.75, 0.75)
ylim.rand <- c(0, 5)
pch.mean  <- 4
show.obs <- TRUE
col.zero <- "darkgrey"

dist.p <- function(parameters, mcmc, main = "") {
  tmp.mean <- apply(mcmc[,parameters], 2, mean)
  plot(density(tmp.mean), xlim = xlim.rand, ylim = ylim.rand, main = "", xlab = "")
  abline(h = 0, col = col.zero)
  points(mean(tmp.mean), 0, pch = pch.mean)
  show(sd(tmp.mean))
}

par(mfrow = c(2, 2))

par(mar = c(2, 4, 7, 2) + 0.1)
dist.p(vars[grepl("^validity:id-valid", vars)], mcmc(m29))
title("Validity", line = 1)
mtext("Density of mean random slopes", font = 2, at = c(14, 1), line = 4, cex = 1.5)

dist.p(vars[grepl("^believability:id-abstract", vars)], mcmc(m29))
title("Believability: Abstract", line = 1)

par(mar = c(4, 4, 5, 2) + 0.1)
dist.p(vars[grepl("^believability:id-believable", vars)], mcmc(m29))
title("Believability: Believable", line = 1)

dist.p(vars[grepl("^believability:id-unbelievable", vars)], mcmc(m29))
title("Believability: Unbelievable", line = 1)

dev.off()

###########################
## Non-Bayesian analysis ##
###########################

require(afex)
nd <- droplevels(subset(d, believability != "abstract"))

(model_na <- mixed(resp ~ validity*believability*exp + (believability*validity|id) + (validity|content), nd, method="KR"))
summary(model_na$full.model)
